﻿using System;

namespace Exceptions
{
	public class Processor
	{
		/// <summary>
		/// Отваря връзка към базата данни.
		/// </summary>
		private void OpenConnection()
		{
			Console.WriteLine("Opening connection...");
		}

		/// <summary>
		/// Извършва работа с базата данни.
		/// </summary>
		/// <exception cref="System.Exception"/>
		private void DoDatabaseStuff()
		{
			Console.WriteLine("Doing database stuff...");

			// Генериране на произволно цяло число
			int randomNumber = new Random().Next(int.MaxValue);
			if (randomNumber % 2 == 0)
				throw new UnluckyNumberException("It's your unlucky day today!", randomNumber);
			if (randomNumber % 3 == 0)
				throw new NotSupportedException("Dude, you're not supported!");

			Console.WriteLine("Done database stuff.");
		}

		/// <summary>
		/// Затваря връзката към база данни.
		/// </summary>
		private void CloseConnection()
		{
			Console.WriteLine("Closing connection...");
		}

		public void Process()
		{
			Console.WriteLine("Processing...");

			OpenConnection();

			try
			{
				DoDatabaseStuff();
			}
			// Прихващане на UnluckyNumberException
			catch (UnluckyNumberException ex)
			{
				Console.WriteLine("Unlucky number ({0}) happened: {1}",
					ex.Number, ex.Message);
			}
			// Прихващане на NotSupportedException
			catch (NotSupportedException ex)
			{
				Console.WriteLine("Not supported. Sorry. {0}", ex.Message);

				CloseConnection();
				// прехвърляне на изключение
				throw;
			}
			// Прихващане на всички изключения (catch-all)
			catch
			{
				// поглъщане на изключение
			}
			// Почистване (независимо дали е възникнало изключение или не)
			finally
			{
				CloseConnection();
			}

			Console.WriteLine("Finished processing...");
		}
	}
}
