﻿using System;

namespace Persons
{
	internal class Program
	{
		/// <summary>
		/// Демонстрира предаване по стойност.
		/// </summary>
		/// <param name="a">Число предавано по стойност</param>
		/// <param name="b">Число предавано по стойност</param>
		/// <remarks>Изводът е, че параметърът предаван по стойност не се променя на мястото, където е извикан методът, въпреки присвояването.</remarks>
		private static void ChangeNumber(int a, int b)
		{
			a = b;
		}

		/// <summary>
		/// Демонстрира предаване по референция.
		/// </summary>
		/// <param name="a">Число предавано по референция</param>
		/// <param name="b">Число предавано по стойност</param>
		/// <remarks>Изводът е, че параметърът предаван по референция се променя на мястото, където е извикан методът.</remarks>
		private static void ChangeNumber2(ref int a, int b)
		{
			a = b;
		}

		/// <summary>
		/// Демонстрира факта, че обектите се предават по референция.
		/// </summary>
		/// <param name="person">Обект от клас Person</param>
		/// <param name="firstName">Стойност, която да се присвои на първото име</param>
		/// <remarks>Изводът е, че не се предава копие на обекта (сякаш е по стойност), а самият той, тъй като името се променя и на мястото на извикване.</remarks>
		private static void ChangeFirstName(NaturalPerson person, string firstName)
		{
			person.FirstName = firstName;
		}

		/// <summary>
		/// Демонстрира използването на изходни параметри.
		/// </summary>
		/// <param name="person">Обект от клас Person</param>
		/// <param name="firstName">Изходен параметър връщаш името на лицето</param>
		/// <param name="lastName">Изходен параметър връщаш фамилията на лицето</param>
		/// <remarks>Изводът е, че изходните параметри могат да се използват за връщане на повече от една стойност там, където е извикан методът.</remarks>
		private static void GetNames(NaturalPerson person, out string firstName, out string lastName)
		{
			firstName = person.FirstName;
			lastName = person.LastName;
		}

		private static void Main(string[] args)
		{
			#region Алтернатива 1. Трудоемко създаване на обект

			//NaturalPerson person = new NaturalPerson();
			//person.FirstName = "Valery";
			//person.MiddleName = "Petrov";
			//person.LastName = "Dachev";

			#endregion

			#region Alternative 2. Създаване на обект с инициализатори

			var person = new NaturalPerson
			{
				FirstName = "Valery",
				MiddleName = "Petrov",
				LastName = "Dachev",
			};

			#endregion

			string fullName = person.GetFullName();
			Console.WriteLine(fullName);
			fullName = person.GetFullName(false);
			Console.WriteLine(fullName);

			person.Introduce();

			#region "ref" keyword

			// Предаването по стойност на "a" не променя стойността му
			int a = 5;
			ChangeNumber(a, 10);

			// Предаването по референция на "a" променя стойността му
			ChangeNumber2(ref a, 15);

			// Обектите винаги се предават по стойност, а не по референция,
			// затова първото име на лицето е променено след извикване на метода.
			ChangeFirstName(person, "Pencho");

			#endregion

			#region "out" parameters

			// Изходящите параметри могат да се използват за
			// получаване на повече от един резултат от извикване
			// на даден метод.
			string firstName;
			string lastName;
			GetNames(person, out firstName, out lastName);

			#endregion

			#region "virtual" keyword

			// Демонстрира често използван подход за прихващане
			// на започването и приключването на операции в
			// класовете наследници.
			person.Save();

			#endregion

			Console.ReadLine();
		}
	}
}
